# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from fireworks import Fireworks, AsyncFireworks
from tests.utils import assert_matches_type
from fireworks.types import (
    EvaluatorGetResponse,
    EvaluatorListResponse,
    EvaluatorCreateResponse,
    EvaluatorUpdateResponse,
    EvaluatorGetUploadEndpointResponse,
    EvaluatorGetBuildLogEndpointResponse,
    EvaluatorGetSourceCodeEndpointResponse,
)
from fireworks.pagination import SyncCursorEvaluators, AsyncCursorEvaluators

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvaluators:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Fireworks) -> None:
        evaluator = client.evaluators.create(
            account_id="account_id",
            evaluator={},
        )
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.create(
            account_id="account_id",
            evaluator={
                "commit_hash": "commitHash",
                "criteria": [
                    {
                        "code_snippets": {
                            "entry_file": "entryFile",
                            "entry_func": "entryFunc",
                            "file_contents": {"foo": "string"},
                            "language": "language",
                        },
                        "description": "description",
                        "name": "name",
                        "type": "TYPE_UNSPECIFIED",
                    }
                ],
                "default_dataset": "defaultDataset",
                "description": "description",
                "display_name": "displayName",
                "entry_point": "entryPoint",
                "requirements": "requirements",
                "source": {
                    "github_repository_name": "githubRepositoryName",
                    "type": "TYPE_UNSPECIFIED",
                },
            },
            evaluator_id="evaluatorId",
        )
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.create(
            account_id="account_id",
            evaluator={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.create(
            account_id="account_id",
            evaluator={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.create(
                account_id="",
                evaluator={},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Fireworks) -> None:
        evaluator = client.evaluators.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
            prepare_code_upload=True,
            commit_hash="commitHash",
            criteria=[
                {
                    "code_snippets": {
                        "entry_file": "entryFile",
                        "entry_func": "entryFunc",
                        "file_contents": {"foo": "string"},
                        "language": "language",
                    },
                    "description": "description",
                    "name": "name",
                    "type": "TYPE_UNSPECIFIED",
                }
            ],
            default_dataset="defaultDataset",
            description="description",
            display_name="displayName",
            entry_point="entryPoint",
            requirements="requirements",
            source={
                "github_repository_name": "githubRepositoryName",
                "type": "TYPE_UNSPECIFIED",
            },
        )
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.update(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.update(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Fireworks) -> None:
        evaluator = client.evaluators.list(
            account_id="account_id",
        )
        assert_matches_type(SyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.list(
            account_id="account_id",
            filter="filter",
            order_by="orderBy",
            page_size=0,
            page_token="pageToken",
            read_mask="readMask",
        )
        assert_matches_type(SyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.list(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(SyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.list(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(SyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.list(
                account_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Fireworks) -> None:
        evaluator = client.evaluators.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(object, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.delete(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.delete(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.get(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.get(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_build_log_endpoint(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_build_log_endpoint_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_build_log_endpoint(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_build_log_endpoint(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_build_log_endpoint(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.get_build_log_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.get_build_log_endpoint(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_source_code_endpoint(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_source_code_endpoint_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_source_code_endpoint(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_source_code_endpoint(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_source_code_endpoint(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.get_source_code_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.get_source_code_endpoint(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_upload_endpoint(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_upload_endpoint_with_all_params(self, client: Fireworks) -> None:
        evaluator = client.evaluators.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_upload_endpoint(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_upload_endpoint(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_upload_endpoint(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.get_upload_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
                filename_to_size={"foo": "string"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.get_upload_endpoint(
                evaluator_id="",
                account_id="account_id",
                filename_to_size={"foo": "string"},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_validate_upload(self, client: Fireworks) -> None:
        evaluator = client.evaluators.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        )
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_validate_upload(self, client: Fireworks) -> None:
        response = client.evaluators.with_raw_response.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = response.parse()
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_validate_upload(self, client: Fireworks) -> None:
        with client.evaluators.with_streaming_response.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = response.parse()
            assert_matches_type(object, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_validate_upload(self, client: Fireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.evaluators.with_raw_response.validate_upload(
                evaluator_id="evaluator_id",
                account_id="",
                body={},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            client.evaluators.with_raw_response.validate_upload(
                evaluator_id="",
                account_id="account_id",
                body={},
            )


class TestAsyncEvaluators:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.create(
            account_id="account_id",
            evaluator={},
        )
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.create(
            account_id="account_id",
            evaluator={
                "commit_hash": "commitHash",
                "criteria": [
                    {
                        "code_snippets": {
                            "entry_file": "entryFile",
                            "entry_func": "entryFunc",
                            "file_contents": {"foo": "string"},
                            "language": "language",
                        },
                        "description": "description",
                        "name": "name",
                        "type": "TYPE_UNSPECIFIED",
                    }
                ],
                "default_dataset": "defaultDataset",
                "description": "description",
                "display_name": "displayName",
                "entry_point": "entryPoint",
                "requirements": "requirements",
                "source": {
                    "github_repository_name": "githubRepositoryName",
                    "type": "TYPE_UNSPECIFIED",
                },
            },
            evaluator_id="evaluatorId",
        )
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.create(
            account_id="account_id",
            evaluator={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.create(
            account_id="account_id",
            evaluator={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorCreateResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.create(
                account_id="",
                evaluator={},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
            prepare_code_upload=True,
            commit_hash="commitHash",
            criteria=[
                {
                    "code_snippets": {
                        "entry_file": "entryFile",
                        "entry_func": "entryFunc",
                        "file_contents": {"foo": "string"},
                        "language": "language",
                    },
                    "description": "description",
                    "name": "name",
                    "type": "TYPE_UNSPECIFIED",
                }
            ],
            default_dataset="defaultDataset",
            description="description",
            display_name="displayName",
            entry_point="entryPoint",
            requirements="requirements",
            source={
                "github_repository_name": "githubRepositoryName",
                "type": "TYPE_UNSPECIFIED",
            },
        )
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.update(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorUpdateResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.update(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.update(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.list(
            account_id="account_id",
        )
        assert_matches_type(AsyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.list(
            account_id="account_id",
            filter="filter",
            order_by="orderBy",
            page_size=0,
            page_token="pageToken",
            read_mask="readMask",
        )
        assert_matches_type(AsyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.list(
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(AsyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.list(
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(AsyncCursorEvaluators[EvaluatorListResponse], evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.list(
                account_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.delete(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(object, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.delete(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.delete(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.get(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorGetResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.get(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.get(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_build_log_endpoint(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_build_log_endpoint_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_build_log_endpoint(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_build_log_endpoint(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.get_build_log_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorGetBuildLogEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_build_log_endpoint(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_build_log_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_build_log_endpoint(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_source_code_endpoint(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_source_code_endpoint_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_source_code_endpoint(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_source_code_endpoint(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.get_source_code_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorGetSourceCodeEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_source_code_endpoint(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_source_code_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_source_code_endpoint(
                evaluator_id="",
                account_id="account_id",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_upload_endpoint_with_all_params(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
            read_mask="readMask",
        )
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.get_upload_endpoint(
            evaluator_id="evaluator_id",
            account_id="account_id",
            filename_to_size={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(EvaluatorGetUploadEndpointResponse, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_upload_endpoint(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_upload_endpoint(
                evaluator_id="evaluator_id",
                account_id="",
                filename_to_size={"foo": "string"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.get_upload_endpoint(
                evaluator_id="",
                account_id="account_id",
                filename_to_size={"foo": "string"},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_validate_upload(self, async_client: AsyncFireworks) -> None:
        evaluator = await async_client.evaluators.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        )
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_validate_upload(self, async_client: AsyncFireworks) -> None:
        response = await async_client.evaluators.with_raw_response.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluator = await response.parse()
        assert_matches_type(object, evaluator, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_validate_upload(self, async_client: AsyncFireworks) -> None:
        async with async_client.evaluators.with_streaming_response.validate_upload(
            evaluator_id="evaluator_id",
            account_id="account_id",
            body={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluator = await response.parse()
            assert_matches_type(object, evaluator, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_validate_upload(self, async_client: AsyncFireworks) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.evaluators.with_raw_response.validate_upload(
                evaluator_id="evaluator_id",
                account_id="",
                body={},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluator_id` but received ''"):
            await async_client.evaluators.with_raw_response.validate_upload(
                evaluator_id="",
                account_id="account_id",
                body={},
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["EvaluationJobGetLogEndpointResponse"]


class EvaluationJobGetLogEndpointResponse(BaseModel):
    """Response carries the stream log URL (for VirtualizedLogViewer).

    Next ID: 4
    """

    content_type: Optional[str] = FieldInfo(alias="contentType", default=None)
    """Content type for the log file (e.g.

    "text/plain"). Only set when execution_log_signed_uri is present.
    """

    execution_log_signed_uri: Optional[str] = FieldInfo(alias="executionLogSignedUri", default=None)
    """
    Short-lived signed URL for the execution log file. Empty if the log file has not
    been created yet (e.g. job not started or still initializing).
    """

    expire_time: Optional[datetime] = FieldInfo(alias="expireTime", default=None)
    """
    Expiration time of the signed URL. Only set when execution_log_signed_uri is
    present.
    """



"""Tests for SDK checkpoint reference resolution behavior."""

from fireworks.training.sdk.client import FiretitanTrainingClient

CROSS_JOB_CHECKPOINT_REF_PREFIX = "cross_job://"


def test_resolve_checkpoint_path_keeps_full_paths_unchanged():
    client = object.__new__(FiretitanTrainingClient)
    full_path = "gs://bucket/path/step-4"
    assert client.resolve_checkpoint_path(full_path) == full_path


def test_resolve_checkpoint_path_returns_name_for_same_job():
    client = object.__new__(FiretitanTrainingClient)
    assert client.resolve_checkpoint_path("step-4") == "step-4"


def test_resolve_checkpoint_path_returns_opaque_ref_for_cross_job():
    client = object.__new__(FiretitanTrainingClient)
    checkpoint_ref = client.resolve_checkpoint_path("step-4", source_job_id="old-job")
    assert checkpoint_ref == f"{CROSS_JOB_CHECKPOINT_REF_PREFIX}old-job/step-4"



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ReinforcementFineTuningStepExecuteParams"]


class ReinforcementFineTuningStepExecuteParams(TypedDict, total=False):
    account_id: str

    dataset: Required[str]
    """Dataset to process for this iteration."""

    output_model: Required[Annotated[str, PropertyInfo(alias="outputModel")]]
    """Output model to materialize when training completes."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .evaluator_source_param import EvaluatorSourceParam

__all__ = ["EvaluatorUpdateParams", "Criterion", "CriterionCodeSnippets"]


class EvaluatorUpdateParams(TypedDict, total=False):
    account_id: str

    prepare_code_upload: Annotated[bool, PropertyInfo(alias="prepareCodeUpload")]
    """
    If true, prepare a new code upload/build attempt by transitioning the evaluator
    to BUILDING state. Can be used without update_mask.
    """

    commit_hash: Annotated[str, PropertyInfo(alias="commitHash")]

    criteria: Iterable[Criterion]

    default_dataset: Annotated[str, PropertyInfo(alias="defaultDataset")]

    description: str

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    entry_point: Annotated[str, PropertyInfo(alias="entryPoint")]

    requirements: str

    source: EvaluatorSourceParam
    """Source information for the evaluator codebase."""


class CriterionCodeSnippets(TypedDict, total=False):
    entry_file: Annotated[str, PropertyInfo(alias="entryFile")]

    entry_func: Annotated[str, PropertyInfo(alias="entryFunc")]

    file_contents: Annotated[Dict[str, str], PropertyInfo(alias="fileContents")]

    language: str


class Criterion(TypedDict, total=False):
    code_snippets: Annotated[CriterionCodeSnippets, PropertyInfo(alias="codeSnippets")]

    description: str

    name: str

    type: Literal["TYPE_UNSPECIFIED", "CODE_SNIPPETS"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from fireworks import Fireworks, AsyncFireworks
from tests.utils import assert_matches_type
from fireworks.types import MessageCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Fireworks) -> None:
        message = client.messages.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        )
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Fireworks) -> None:
        message = client.messages.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
            max_tokens=1024,
            metadata={"user_id": "13803d75-b4b5-4c3e-b2a2-6f21399b021b"},
            output_config={
                "effort": "low",
                "format": {
                    "schema": {"foo": "bar"},
                    "type": "json_schema",
                },
            },
            raw_output=True,
            stop_sequences=["string"],
            stream=True,
            system=[
                {
                    "text": "Today's date is 2024-06-01.",
                    "type": "text",
                    "cache_control": {
                        "type": "ephemeral",
                        "ttl": "5m",
                    },
                    "citations": [
                        {
                            "cited_text": "cited_text",
                            "document_index": 0,
                            "document_title": "x",
                            "end_char_index": 0,
                            "start_char_index": 0,
                            "type": "char_location",
                        }
                    ],
                }
            ],
            temperature=1,
            thinking={
                "budget_tokens": 1024,
                "type": "enabled",
            },
            tool_choice={
                "type": "auto",
                "disable_parallel_tool_use": True,
            },
            tools=[
                {
                    "input_schema": {
                        "type": "object",
                        "properties": {
                            "location": "bar",
                            "unit": "bar",
                        },
                        "required": ["location"],
                    },
                    "name": "name",
                    "description": "Get the current weather in a given location",
                    "strict": True,
                    "type": "custom",
                }
            ],
            top_k=5,
            top_p=0.7,
        )
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Fireworks) -> None:
        response = client.messages.with_raw_response.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Fireworks) -> None:
        with client.messages.with_streaming_response.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageCreateResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMessages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncFireworks) -> None:
        message = await async_client.messages.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        )
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncFireworks) -> None:
        message = await async_client.messages.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
            max_tokens=1024,
            metadata={"user_id": "13803d75-b4b5-4c3e-b2a2-6f21399b021b"},
            output_config={
                "effort": "low",
                "format": {
                    "schema": {"foo": "bar"},
                    "type": "json_schema",
                },
            },
            raw_output=True,
            stop_sequences=["string"],
            stream=True,
            system=[
                {
                    "text": "Today's date is 2024-06-01.",
                    "type": "text",
                    "cache_control": {
                        "type": "ephemeral",
                        "ttl": "5m",
                    },
                    "citations": [
                        {
                            "cited_text": "cited_text",
                            "document_index": 0,
                            "document_title": "x",
                            "end_char_index": 0,
                            "start_char_index": 0,
                            "type": "char_location",
                        }
                    ],
                }
            ],
            temperature=1,
            thinking={
                "budget_tokens": 1024,
                "type": "enabled",
            },
            tool_choice={
                "type": "auto",
                "disable_parallel_tool_use": True,
            },
            tools=[
                {
                    "input_schema": {
                        "type": "object",
                        "properties": {
                            "location": "bar",
                            "unit": "bar",
                        },
                        "required": ["location"],
                    },
                    "name": "name",
                    "description": "Get the current weather in a given location",
                    "strict": True,
                    "type": "custom",
                }
            ],
            top_k=5,
            top_p=0.7,
        )
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncFireworks) -> None:
        response = await async_client.messages.with_raw_response.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageCreateResponse, message, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncFireworks) -> None:
        async with async_client.messages.with_streaming_response.create(
            messages=[
                {
                    "content": "Hello, world",
                    "role": "user",
                }
            ],
            model="claude-opus-4-6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageCreateResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel

__all__ = ["LogProbs"]


class LogProbs(BaseModel):
    """Legacy log probabilities format"""

    text_offset: Optional[List[int]] = None

    token_ids: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None



from pathlib import Path
from tempfile import NamedTemporaryFile

import nox

_TRAINING_ONLY_PACKAGES = ("tinker==", "tinker-cookbook==")


def _install_dev_deps_without_training_extras(session: nox.Session) -> None:
    """Install lockfile deps while skipping training-only extras.

    `test-pydantic-v1` runs on Python 3.9 to validate base SDK compatibility.
    Training extras (`tinker`, `tinker-cookbook`) currently require newer
    Python versions, so we exclude only those packages from the shared dev
    lockfile for this session.
    """
    lockfile_lines = Path("requirements-dev.lock").read_text(encoding="utf-8").splitlines()
    filtered_lines = [
        line for line in lockfile_lines if not line.strip().startswith(_TRAINING_ONLY_PACKAGES)
    ]

    with NamedTemporaryFile(mode="w", encoding="utf-8", suffix=".txt", delete=False) as temp_requirements:
        temp_requirements.write("\n".join(filtered_lines) + "\n")
        temp_requirements_path = temp_requirements.name

    session.install("-r", temp_requirements_path)


@nox.session(reuse_venv=True, name="test-pydantic-v1")
def test_pydantic_v1(session: nox.Session) -> None:
    _install_dev_deps_without_training_extras(session)
    session.install("pydantic<2")

    session.run("pytest", "--showlocals", "--ignore=tests/functional", *session.posargs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import dpo_job_get_params, dpo_job_list_params, dpo_job_create_params, dpo_job_resume_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorDpoJobs, AsyncCursorDpoJobs
from .._base_client import AsyncPaginator, make_request_options
from ..types.dpo_job import DpoJob
from ..types.shared_params.wandb_config import WandbConfig
from ..types.shared_params.training_config import TrainingConfig
from ..types.dpo_job_get_metrics_file_endpoint_response import DpoJobGetMetricsFileEndpointResponse
from ..types.shared_params.reinforcement_learning_loss_config import ReinforcementLearningLossConfig

__all__ = ["DpoJobsResource", "AsyncDpoJobsResource"]


class DpoJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DpoJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return DpoJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DpoJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return DpoJobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str | None = None,
        dataset: str,
        dpo_job_id: str | Omit = omit,
        aws_s3_config: dpo_job_create_params.AwsS3Config | Omit = omit,
        azure_blob_storage_config: dpo_job_create_params.AzureBlobStorageConfig | Omit = omit,
        display_name: str | Omit = omit,
        loss_config: ReinforcementLearningLossConfig | Omit = omit,
        training_config: TrainingConfig | Omit = omit,
        wandb_config: WandbConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """
        Args:
          dataset: The name of the dataset used for training.

          dpo_job_id: ID of the DPO job, a random ID will be generated if not specified.

          aws_s3_config: The AWS configuration for S3 dataset access.

          azure_blob_storage_config: The Azure configuration for Azure Blob Storage dataset access.

          loss_config: Loss configuration for the training job. If not specified, defaults to DPO loss.
              Set method to ORPO for ORPO training.

          training_config: Common training configurations.

          wandb_config: The Weights & Biases team/user account for logging job progress.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/dpoJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs",
            body=maybe_transform(
                {
                    "dataset": dataset,
                    "aws_s3_config": aws_s3_config,
                    "azure_blob_storage_config": azure_blob_storage_config,
                    "display_name": display_name,
                    "loss_config": loss_config,
                    "training_config": training_config,
                    "wandb_config": wandb_config,
                },
                dpo_job_create_params.DpoJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dpo_job_id": dpo_job_id}, dpo_job_create_params.DpoJobCreateParams),
            ),
            cast_to=DpoJob,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorDpoJobs[DpoJob]:
        """Args:
          filter: Filter criteria for the returned jobs.

        See https://google.aip.dev/160 for the
              filter syntax specification.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of dpo jobs to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListDpoJobs call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListDpoJobs must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/dpoJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs",
            page=SyncCursorDpoJobs[DpoJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    dpo_job_list_params.DpoJobListParams,
                ),
            ),
            model=DpoJob,
        )

    def delete(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return self._delete(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"read_mask": read_mask}, dpo_job_get_params.DpoJobGetParams),
            ),
            cast_to=DpoJob,
        )

    def get_metrics_file_endpoint(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJobGetMetricsFileEndpointResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:getMetricsFileEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:getMetricsFileEndpoint",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DpoJobGetMetricsFileEndpointResponse,
        )

    def resume(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """
        Resume Dpo Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:resume"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:resume",
            body=maybe_transform(body, dpo_job_resume_params.DpoJobResumeParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DpoJob,
        )


class AsyncDpoJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDpoJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDpoJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDpoJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncDpoJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str | None = None,
        dataset: str,
        dpo_job_id: str | Omit = omit,
        aws_s3_config: dpo_job_create_params.AwsS3Config | Omit = omit,
        azure_blob_storage_config: dpo_job_create_params.AzureBlobStorageConfig | Omit = omit,
        display_name: str | Omit = omit,
        loss_config: ReinforcementLearningLossConfig | Omit = omit,
        training_config: TrainingConfig | Omit = omit,
        wandb_config: WandbConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """
        Args:
          dataset: The name of the dataset used for training.

          dpo_job_id: ID of the DPO job, a random ID will be generated if not specified.

          aws_s3_config: The AWS configuration for S3 dataset access.

          azure_blob_storage_config: The Azure configuration for Azure Blob Storage dataset access.

          loss_config: Loss configuration for the training job. If not specified, defaults to DPO loss.
              Set method to ORPO for ORPO training.

          training_config: Common training configurations.

          wandb_config: The Weights & Biases team/user account for logging job progress.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/dpoJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs",
            body=await async_maybe_transform(
                {
                    "dataset": dataset,
                    "aws_s3_config": aws_s3_config,
                    "azure_blob_storage_config": azure_blob_storage_config,
                    "display_name": display_name,
                    "loss_config": loss_config,
                    "training_config": training_config,
                    "wandb_config": wandb_config,
                },
                dpo_job_create_params.DpoJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"dpo_job_id": dpo_job_id}, dpo_job_create_params.DpoJobCreateParams),
            ),
            cast_to=DpoJob,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DpoJob, AsyncCursorDpoJobs[DpoJob]]:
        """Args:
          filter: Filter criteria for the returned jobs.

        See https://google.aip.dev/160 for the
              filter syntax specification.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of dpo jobs to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListDpoJobs call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListDpoJobs must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/dpoJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs",
            page=AsyncCursorDpoJobs[DpoJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    dpo_job_list_params.DpoJobListParams,
                ),
            ),
            model=DpoJob,
        )

    async def delete(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return await self._delete(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"read_mask": read_mask}, dpo_job_get_params.DpoJobGetParams),
            ),
            cast_to=DpoJob,
        )

    async def get_metrics_file_endpoint(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJobGetMetricsFileEndpointResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:getMetricsFileEndpoint"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:getMetricsFileEndpoint",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DpoJobGetMetricsFileEndpointResponse,
        )

    async def resume(
        self,
        dpo_job_id: str,
        *,
        account_id: str | None = None,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DpoJob:
        """
        Resume Dpo Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dpo_job_id:
            raise ValueError(f"Expected a non-empty value for `dpo_job_id` but received {dpo_job_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:resume"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/dpoJobs/{dpo_job_id}:resume",
            body=await async_maybe_transform(body, dpo_job_resume_params.DpoJobResumeParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DpoJob,
        )


class DpoJobsResourceWithRawResponse:
    def __init__(self, dpo_jobs: DpoJobsResource) -> None:
        self._dpo_jobs = dpo_jobs

        self.create = to_raw_response_wrapper(
            dpo_jobs.create,
        )
        self.list = to_raw_response_wrapper(
            dpo_jobs.list,
        )
        self.delete = to_raw_response_wrapper(
            dpo_jobs.delete,
        )
        self.get = to_raw_response_wrapper(
            dpo_jobs.get,
        )
        self.get_metrics_file_endpoint = to_raw_response_wrapper(
            dpo_jobs.get_metrics_file_endpoint,
        )
        self.resume = to_raw_response_wrapper(
            dpo_jobs.resume,
        )


class AsyncDpoJobsResourceWithRawResponse:
    def __init__(self, dpo_jobs: AsyncDpoJobsResource) -> None:
        self._dpo_jobs = dpo_jobs

        self.create = async_to_raw_response_wrapper(
            dpo_jobs.create,
        )
        self.list = async_to_raw_response_wrapper(
            dpo_jobs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            dpo_jobs.delete,
        )
        self.get = async_to_raw_response_wrapper(
            dpo_jobs.get,
        )
        self.get_metrics_file_endpoint = async_to_raw_response_wrapper(
            dpo_jobs.get_metrics_file_endpoint,
        )
        self.resume = async_to_raw_response_wrapper(
            dpo_jobs.resume,
        )


class DpoJobsResourceWithStreamingResponse:
    def __init__(self, dpo_jobs: DpoJobsResource) -> None:
        self._dpo_jobs = dpo_jobs

        self.create = to_streamed_response_wrapper(
            dpo_jobs.create,
        )
        self.list = to_streamed_response_wrapper(
            dpo_jobs.list,
        )
        self.delete = to_streamed_response_wrapper(
            dpo_jobs.delete,
        )
        self.get = to_streamed_response_wrapper(
            dpo_jobs.get,
        )
        self.get_metrics_file_endpoint = to_streamed_response_wrapper(
            dpo_jobs.get_metrics_file_endpoint,
        )
        self.resume = to_streamed_response_wrapper(
            dpo_jobs.resume,
        )


class AsyncDpoJobsResourceWithStreamingResponse:
    def __init__(self, dpo_jobs: AsyncDpoJobsResource) -> None:
        self._dpo_jobs = dpo_jobs

        self.create = async_to_streamed_response_wrapper(
            dpo_jobs.create,
        )
        self.list = async_to_streamed_response_wrapper(
            dpo_jobs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            dpo_jobs.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            dpo_jobs.get,
        )
        self.get_metrics_file_endpoint = async_to_streamed_response_wrapper(
            dpo_jobs.get_metrics_file_endpoint,
        )
        self.resume = async_to_streamed_response_wrapper(
            dpo_jobs.resume,
        )



"""Tests for fireworks.training.sdk.trainer — payload serialization and state machine logic."""

from __future__ import annotations

from unittest.mock import MagicMock, patch

import pytest

from fireworks.training.sdk.trainer import (
    TrainerJobConfig,
    TrainerJobManager,
    TrainingShapeProfile,
    TrainerServiceEndpoint,
)


@pytest.fixture
def mgr():
    return TrainerJobManager(
        api_key="test-key",
        account_id="test-account",
        base_url="https://api.example.com",
        additional_headers={"X-Custom": "val"},
    )


@pytest.fixture
def basic_config():
    return TrainerJobConfig(
        base_model="accounts/test/models/qwen3-1p7b",
        node_count=2,
        region="US_OHIO_1",
        lora_rank=0,
        max_context_length=4096,
        learning_rate=1e-5,
        skip_validations=True,
        hot_load_deployment_id="my-deploy",
    )


class TestCreate:
    @patch("fireworks.training.sdk.trainer.request_with_retries")
    def test_payload_construction(self, mock_req, mgr, basic_config):
        resp = MagicMock()
        resp.ok = True
        resp.status_code = 200
        resp.json.return_value = {"name": "accounts/test/rlorTrainerJobs/job-1"}
        mock_req.return_value = resp

        result = mgr._create(basic_config)

        call_kwargs = mock_req.call_args
        url = call_kwargs[0][1]
        payload = call_kwargs[1]["json"]

        assert "deploymentId=my-deploy" in url
        assert "skipValidations=true" in url
        assert payload["serviceMode"] is True
        assert payload["nodeCount"] == 2
        assert payload["trainingConfig"]["baseModel"] == "accounts/test/models/qwen3-1p7b"
        assert payload["trainingConfig"]["region"] == "US_OHIO_1"
        assert payload["hotLoadDeploymentId"] == "my-deploy"

    @patch("fireworks.training.sdk.trainer.request_with_retries")
    def test_validated_shape_omits_all_infra_fields(self, mock_req, mgr):
        """Validated-shape path sends only algorithm fields; all shape-derived
        infra fields are omitted so the backend populates them from the shape."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            training_shape_ref="accounts/test-account/trainingShapes/ts-test/versions/shape-v1",
            accelerator_type="NVIDIA_H100_80GB",
            accelerator_count=8,
            custom_image_tag="0.33.0",
            node_count=4,
            max_context_length=8192,
            region="US_OHIO_1",
            extra_args=["--flag"],
        )
        resp = MagicMock()
        resp.ok = True
        resp.status_code = 200
        resp.json.return_value = {"name": "j"}
        mock_req.return_value = resp

        mgr._create(config)

        url = mock_req.call_args[0][1]
        payload = mock_req.call_args[1]["json"]
        assert "trainingShape=" in url
        assert "accounts%2Ftest-account%2FtrainingShapes%2Fts-test%2Fversions%2Fshape-v1" in url
        tc = payload["trainingConfig"]
        assert "acceleratorType" not in tc
        assert "acceleratorCount" not in tc
        assert "customImageTag" not in tc
        assert "maxContextLength" not in tc
        assert "nodeCount" not in payload
        assert tc["region"] == "US_OHIO_1"
        assert tc["extraArgs"] == ["--flag"]

    @patch("fireworks.training.sdk.trainer.request_with_retries")
    def test_skip_validations_sends_all_fields(self, mock_req, mgr):
        """Skip-validation path sends all fields including shape-derived ones."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            training_shape_ref="accounts/test-account/trainingShapes/ts-test/versions/shape-v1",
            skip_validations=True,
            accelerator_type="NVIDIA_H100_80GB",
            accelerator_count=8,
            custom_image_tag="0.33.0",
            node_count=4,
            max_context_length=8192,
            region="US_OHIO_1",
        )
        resp = MagicMock()
        resp.ok = True
        resp.status_code = 200
        resp.json.return_value = {"name": "j"}
        mock_req.return_value = resp

        mgr._create(config)

        url = mock_req.call_args[0][1]
        payload = mock_req.call_args[1]["json"]
        assert "trainingShape=" in url
        assert "skipValidations=true" in url
        tc = payload["trainingConfig"]
        assert tc["acceleratorType"] == "NVIDIA_H100_80GB"
        assert tc["acceleratorCount"] == 8
        assert tc["customImageTag"] == "0.33.0"
        assert tc["maxContextLength"] == 8192
        assert payload["nodeCount"] == 4
        assert tc["region"] == "US_OHIO_1"

    @patch("fireworks.training.sdk.trainer.request_with_retries")
    def test_extra_args_flattened(self, mock_req, mgr):
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            extra_args=["--pp 8", "--ep=4", "--flag"],
        )
        resp = MagicMock()
        resp.ok = True
        resp.status_code = 200
        resp.json.return_value = {"name": "j"}
        mock_req.return_value = resp

        mgr._create(config)
        payload = mock_req.call_args[1]["json"]
        assert payload["trainingConfig"]["extraArgs"] == [
            "--pp",
            "8",
            "--ep=4",
            "--flag",
        ]


class TestPollUntilReady:
    @patch.object(TrainerJobManager, "_check_healthz", return_value=True)
    @patch.object(TrainerJobManager, "get")
    def test_running_uses_gateway_endpoint(self, mock_get, mock_healthz, mgr):
        mock_get.return_value = {
            "state": "JOB_STATE_RUNNING",
            "directRouteHandle": "https://trainer.internal:8080",
        }
        result = mgr._poll_until_ready("job-1", "accounts/test/rlorTrainerJobs/job-1", timeout_s=10)
        assert isinstance(result, TrainerServiceEndpoint)
        assert result.job_id == "job-1"
        assert result.base_url == "https://api.example.com/training/v1/rlorTrainerJobs/test-account/job-1"

    @patch.object(TrainerJobManager, "get")
    def test_failed_raises_runtime_error(self, mock_get, mgr):
        mock_get.return_value = {
            "state": "JOB_STATE_FAILED",
            "status": {"message": "GPU OOM"},
        }
        with pytest.raises(RuntimeError, match="failed"):
            mgr._poll_until_ready("job-1", "name", timeout_s=10)

    @patch("fireworks.training.sdk.trainer.time.sleep")
    @patch("fireworks.training.sdk.trainer.time.time")
    @patch.object(TrainerJobManager, "get")
    def test_timeout_raises(self, mock_get, mock_time, mock_sleep, mgr):
        # Use a call counter instead of a fixed list: return 0 for the first
        # several calls, then jump past the timeout.  This avoids StopIteration
        # from the logging module's internal time.time() calls consuming values.
        call_count = 0

        def fake_time():
            nonlocal call_count
            call_count += 1
            return 0.0 if call_count <= 6 else 100.0

        mock_time.side_effect = fake_time
        mock_get.return_value = {"state": "JOB_STATE_CREATING"}
        with pytest.raises(TimeoutError, match="not become ready"):
            mgr._poll_until_ready("job-1", "name", timeout_s=5)


class TestReconnectAndWait:
    @patch.object(TrainerJobManager, "resume_and_wait")
    @patch.object(TrainerJobManager, "get")
    def test_failed_triggers_resume(self, mock_get, mock_resume, mgr):
        mock_get.return_value = {"state": "JOB_STATE_FAILED"}
        ep = TrainerServiceEndpoint("n", "j", "https://u")
        mock_resume.return_value = ep
        result = mgr.reconnect_and_wait("j")
        assert result is ep

    @patch("fireworks.training.sdk.trainer.time.sleep")
    @patch("fireworks.training.sdk.trainer.time.time")
    @patch.object(TrainerJobManager, "get")
    def test_stuck_state_raises(self, mock_get, mock_time, mock_sleep, mgr):
        call_count = 0

        def fake_time():
            nonlocal call_count
            call_count += 1
            return 0.0 if call_count <= 4 else 200.0

        mock_time.side_effect = fake_time
        mock_get.return_value = {"state": "JOB_STATE_CREATING"}
        with pytest.raises(RuntimeError, match="stuck"):
            mgr.reconnect_and_wait("j", max_wait_for_resumable_s=5)


class TestResolveTrainingProfile:
    def test_parses_sharding(self):
        """resolve_training_profile parses the latest shape-version snapshot."""
        mgr = TrainerJobManager(api_key="k", account_id="a", base_url="https://x")
        with patch("fireworks.training.sdk.trainer.request_with_retries") as mock_req:
            resp = MagicMock()
            resp.ok = True
            resp.json.return_value = {
                "trainingShapeVersions": [
                    {
                        "name": "accounts/a/trainingShapes/ts-test/versions/ver-123",
                        "snapshot": {
                            "name": "accounts/a/trainingShapes/ts-test",
                            "trainerImageTag": "0.33.0",
                            "maxSupportedContextLength": 8192,
                            "nodeCount": 2,
                            "deploymentShapeVersion": "dsv",
                            "deploymentImageTag": "img",
                            "acceleratorType": "NVIDIA_H100_80GB",
                            "acceleratorCount": 8,
                            "baseModelWeightPrecision": "bfloat16",
                            "trainerShardingScheme": {
                                "tensorParallelism": 1,
                                "pipelineParallelism": 4,
                                "contextParallelism": 1,
                                "expertParallelism": 1,
                            },
                        },
                    },
                ],
            }
            mock_req.return_value = resp
            profile = mgr.resolve_training_profile("ts-test")
            called_url = mock_req.call_args[0][1]
            assert called_url.endswith("/trainingShapes/ts-test/versions?filter=latest_validated%3Dtrue&pageSize=1")
            assert profile.pipeline_parallelism == 4
            assert profile.max_supported_context_length == 8192
            assert profile.training_shape_version == ("accounts/a/trainingShapes/ts-test/versions/ver-123")
            assert profile.training_shape == "accounts/a/trainingShapes/ts-test"


# ---------------------------------------------------------------------------
# TrainingShapeProfile.training_shape / deployment_shape
# ---------------------------------------------------------------------------


def _make_profile(**overrides) -> TrainingShapeProfile:
    defaults = dict(
        training_shape_version="accounts/fw/trainingShapes/ts-x/versions/1",
        trainer_image_tag="0.33.0",
        max_supported_context_length=8192,
        node_count=2,
        deployment_shape_version="accounts/fw/deploymentShapes/ds-x/versions/1",
        deployment_image_tag="img",
        accelerator_type="NVIDIA_H100_80GB",
        accelerator_count=8,
        base_model_weight_precision="bfloat16",
        pipeline_parallelism=1,
    )
    defaults.update(overrides)
    return TrainingShapeProfile(**defaults)


class TestTrainingShapeProperty:
    def test_strips_version_suffix(self):
        profile = _make_profile(
            training_shape_version="accounts/fw/trainingShapes/ts-x/versions/1",
        )
        assert profile.training_shape == "accounts/fw/trainingShapes/ts-x"

    def test_empty_returns_none(self):
        profile = _make_profile(training_shape_version="")
        assert profile.training_shape is None

    def test_no_version_suffix_unchanged(self):
        profile = _make_profile(
            training_shape_version="accounts/fw/trainingShapes/ts-x",
        )
        assert profile.training_shape == "accounts/fw/trainingShapes/ts-x"


# ---------------------------------------------------------------------------
# TrainingShapeProfile.deployment_shape
# ---------------------------------------------------------------------------


class TestDeploymentShapeProperty:
    def test_strips_version_suffix(self):
        profile = _make_profile(
            deployment_shape_version="accounts/fw/deploymentShapes/ds-x/versions/1",
        )
        assert profile.deployment_shape == "accounts/fw/deploymentShapes/ds-x"

    def test_empty_returns_none(self):
        profile = _make_profile(deployment_shape_version="")
        assert profile.deployment_shape is None

    def test_no_version_suffix_unchanged(self):
        profile = _make_profile(deployment_shape_version="accounts/fw/deploymentShapes/ds-x")
        assert profile.deployment_shape == "accounts/fw/deploymentShapes/ds-x"


# ---------------------------------------------------------------------------
# TrainerJobConfig.apply_shape
# ---------------------------------------------------------------------------


class TestApplyShape:
    def test_shape_values_applied(self):
        """Shape values populate the config."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
        )
        profile = _make_profile()
        config.apply_shape(profile)

        assert config.accelerator_type == "NVIDIA_H100_80GB"
        assert config.accelerator_count == 8
        assert config.custom_image_tag == "0.33.0"
        assert config.node_count == 2
        assert config.max_context_length == 8192

    def test_shape_wins_without_skip_validations_for_copied_fields(self):
        """Without skip_validations, shape wins for copied launch fields."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            accelerator_type="NVIDIA_A100_80GB",
            accelerator_count=4,
            max_context_length=4096,
        )
        profile = _make_profile()
        config.apply_shape(profile)

        assert config.accelerator_type == "NVIDIA_H100_80GB"
        assert config.accelerator_count == 8
        assert config.node_count == 2
        assert config.max_context_length == 8192

    def test_skip_validations_keeps_user_overrides(self):
        """With skip_validations, user values override copied shape fields."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            skip_validations=True,
            accelerator_type="NVIDIA_A100_80GB",
            accelerator_count=4,
            max_context_length=4096,
        )
        profile = _make_profile()
        config.apply_shape(profile)

        assert config.accelerator_type == "NVIDIA_A100_80GB"
        assert config.accelerator_count == 4
        assert config.max_context_length == 4096
        assert config.custom_image_tag == "0.33.0"
        assert config.node_count == 2

    def test_skip_validations_fills_none_fields(self):
        """With skip_validations, shape fills fields the user left as None."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            skip_validations=True,
        )
        profile = _make_profile()
        config.apply_shape(profile)

        assert config.accelerator_type == "NVIDIA_H100_80GB"
        assert config.accelerator_count == 8
        assert config.custom_image_tag == "0.33.0"
        assert config.max_context_length == 8192

    def test_skip_validations_can_partially_override_accelerators(self):
        """Skip-validation launches can override part of the accelerator tuple."""
        config = TrainerJobConfig(
            base_model="accounts/test/models/m",
            skip_validations=True,
            accelerator_type="MY_ACCEL",
        )
        profile = _make_profile()
        config.apply_shape(profile)

        assert config.accelerator_type == "MY_ACCEL"
        assert config.accelerator_count == 8



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .shared_params.wandb_config import WandbConfig
from .shared_params.training_config import TrainingConfig
from .shared_params.reinforcement_learning_loss_config import ReinforcementLearningLossConfig

__all__ = ["ReinforcementFineTuningStepCreateParams", "AwsS3Config", "AzureBlobStorageConfig"]


class ReinforcementFineTuningStepCreateParams(TypedDict, total=False):
    account_id: str

    rlor_trainer_job_id: Annotated[str, PropertyInfo(alias="rlorTrainerJobId")]
    """ID of the RLOR trainer job, a random UUID will be generated if not specified."""

    aws_s3_config: Annotated[AwsS3Config, PropertyInfo(alias="awsS3Config")]
    """The AWS configuration for S3 dataset access."""

    azure_blob_storage_config: Annotated[AzureBlobStorageConfig, PropertyInfo(alias="azureBlobStorageConfig")]
    """The Azure configuration for Azure Blob Storage dataset access."""

    dataset: str
    """The name of the dataset used for training."""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    eval_auto_carveout: Annotated[bool, PropertyInfo(alias="evalAutoCarveout")]
    """Whether to auto-carve the dataset for eval."""

    evaluation_dataset: Annotated[str, PropertyInfo(alias="evaluationDataset")]
    """The name of a separate dataset to use for evaluation."""

    forward_only: Annotated[bool, PropertyInfo(alias="forwardOnly")]
    """
    When true, run the trainer in forward-only mode (no backward/optimizer). Used
    for reference models in GRPO that only need forward passes.
    """

    hot_load_deployment_id: Annotated[str, PropertyInfo(alias="hotLoadDeploymentId")]
    """The deployment ID used for hot loading.

    When set, checkpoints are saved to this deployment's hot load bucket, enabling
    weight swaps on inference. Only valid for service-mode or keep-alive jobs.
    """

    keep_alive: Annotated[bool, PropertyInfo(alias="keepAlive")]

    loss_config: Annotated[ReinforcementLearningLossConfig, PropertyInfo(alias="lossConfig")]
    """
    Reinforcement learning loss method + hyperparameters for the underlying trainer.
    """

    node_count: Annotated[int, PropertyInfo(alias="nodeCount")]
    """
    The number of nodes to use for the fine-tuning job. If not specified, the
    default is 1.
    """

    reward_weights: Annotated[SequenceNotStr[str], PropertyInfo(alias="rewardWeights")]
    """
    A list of reward metrics to use for training in format of
    "<reward_name>=<weight>".
    """

    rollout_deployment_name: Annotated[str, PropertyInfo(alias="rolloutDeploymentName")]
    """Rollout deployment name associated with this RLOR trainer job. This is optional.

    If not set, trainer will not trigger weight sync to rollout engine.
    """

    service_mode: Annotated[bool, PropertyInfo(alias="serviceMode")]
    """Service-mode RLOR trainers currently support full-parameter tuning only.

    When enabled, `trainingConfig.loraRank` must be 0 (`loraRank>0` is rejected).
    """

    training_config: Annotated[TrainingConfig, PropertyInfo(alias="trainingConfig")]
    """Common training configurations."""

    use_purpose: Annotated[str, PropertyInfo(alias="usePurpose")]
    """Use dedicated resources for the job.

    The only supported value currently is "pilot". Defaults to empty.
    """

    wandb_config: Annotated[WandbConfig, PropertyInfo(alias="wandbConfig")]
    """The Weights & Biases team/user account for logging training progress."""


class AwsS3Config(TypedDict, total=False):
    """The AWS configuration for S3 dataset access."""

    credentials_secret: Annotated[str, PropertyInfo(alias="credentialsSecret")]

    iam_role_arn: Annotated[str, PropertyInfo(alias="iamRoleArn")]


class AzureBlobStorageConfig(TypedDict, total=False):
    """The Azure configuration for Azure Blob Storage dataset access."""

    credentials_secret: Annotated[str, PropertyInfo(alias="credentialsSecret")]
    """
    Reference to a Secret resource containing Azure credentials. Format:
    accounts/{account_id}/secrets/{secret_id} The secret value must be JSON:
    {"connection_string": "..."} or {"sas_token": "..."} or {"account_key": "..."}
    Mutually exclusive with managed_identity_client_id.
    """

    managed_identity_client_id: Annotated[str, PropertyInfo(alias="managedIdentityClientId")]
    """
    Managed Identity Client ID for GCP-to-Azure Workload Identity Federation.
    Format: uuid Mutually exclusive with credentials_secret.
    """

    tenant_id: Annotated[str, PropertyInfo(alias="tenantId")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .type_date import TypeDate
from .peft_details import PeftDetails
from .shared.status import Status
from .base_model_details import BaseModelDetails
from .conversation_config import ConversationConfig
from .shared.deployed_model_ref import DeployedModelRef

__all__ = ["Model"]


class Model(BaseModel):
    base_model_details: Optional[BaseModelDetails] = FieldInfo(alias="baseModelDetails", default=None)
    """Base model details. Required if kind is HF_BASE_MODEL.

    Must not be set otherwise.
    """

    calibrated: Optional[bool] = None
    """If true, the model is calibrated and can be deployed to non-FP16 precisions."""

    cluster: Optional[str] = None
    """The resource name of the BYOC cluster to which this model belongs. e.g.

    accounts/my-account/clusters/my-cluster. Empty if it belongs to a Fireworks
    cluster.
    """

    context_length: Optional[int] = FieldInfo(alias="contextLength", default=None)
    """The maximum context length supported by the model."""

    conversation_config: Optional[ConversationConfig] = FieldInfo(alias="conversationConfig", default=None)
    """If set, the Chat Completions API will be enabled for this model."""

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the model."""

    default_draft_model: Optional[str] = FieldInfo(alias="defaultDraftModel", default=None)
    """The default draft model to use when creating a deployment.

    If empty, speculative decoding is disabled by default.
    """

    default_draft_token_count: Optional[int] = FieldInfo(alias="defaultDraftTokenCount", default=None)
    """
    The default draft token count to use when creating a deployment. Must be
    specified if default_draft_model is specified.
    """

    default_sampling_params: Optional[Dict[str, float]] = FieldInfo(alias="defaultSamplingParams", default=None)
    """A json object that contains the default sampling parameters for the model."""

    deployed_model_refs: Optional[List[DeployedModelRef]] = FieldInfo(alias="deployedModelRefs", default=None)
    """Populated from GetModel API call only."""

    deprecation_date: Optional[TypeDate] = FieldInfo(alias="deprecationDate", default=None)
    """
    If specified, this is the date when the serverless deployment of the model will
    be taken down.
    """

    description: Optional[str] = None
    """The description of the model. Must be fewer than 1000 characters long."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Human-readable display name of the model.

    e.g. "My Model" Must be fewer than 64 characters long.
    """

    fine_tuning_job: Optional[str] = FieldInfo(alias="fineTuningJob", default=None)
    """
    If the model was created from a fine-tuning job, this is the fine-tuning job
    name.
    """

    github_url: Optional[str] = FieldInfo(alias="githubUrl", default=None)
    """The URL to GitHub repository of the model."""

    hugging_face_url: Optional[str] = FieldInfo(alias="huggingFaceUrl", default=None)
    """The URL to the Hugging Face model."""

    imported_from: Optional[str] = FieldInfo(alias="importedFrom", default=None)
    """The name of the the model from which this was imported.

    This field is empty if the model was not imported.
    """

    kind: Optional[
        Literal[
            "KIND_UNSPECIFIED",
            "HF_BASE_MODEL",
            "HF_PEFT_ADDON",
            "HF_TEFT_ADDON",
            "FLUMINA_BASE_MODEL",
            "FLUMINA_ADDON",
            "DRAFT_ADDON",
            "FIRE_AGENT",
            "LIVE_MERGE",
            "CUSTOM_MODEL",
            "EMBEDDING_MODEL",
            "SNAPSHOT_MODEL",
        ]
    ] = None
    """The kind of model. If not specified, the default is HF_PEFT_ADDON."""

    name: Optional[str] = None

    peft_details: Optional[PeftDetails] = FieldInfo(alias="peftDetails", default=None)
    """PEFT addon details. Required if kind is HF_PEFT_ADDON or HF_TEFT_ADDON."""

    public: Optional[bool] = None
    """If true, the model will be publicly readable."""

    rl_tunable: Optional[bool] = FieldInfo(alias="rlTunable", default=None)
    """If true, the model is RL tunable."""

    snapshot_type: Optional[Literal["FULL_SNAPSHOT", "INCREMENTAL_SNAPSHOT"]] = FieldInfo(
        alias="snapshotType", default=None
    )

    state: Optional[Literal["STATE_UNSPECIFIED", "UPLOADING", "READY"]] = None
    """The state of the model."""

    status: Optional[Status] = None
    """Contains detailed message when the last model operation fails."""

    supported_precisions: Optional[
        List[
            Literal[
                "PRECISION_UNSPECIFIED",
                "FP16",
                "FP8",
                "FP8_MM",
                "FP8_AR",
                "FP8_MM_KV_ATTN",
                "FP8_KV",
                "FP8_MM_V2",
                "FP8_V2",
                "FP8_MM_KV_ATTN_V2",
                "NF4",
                "FP4",
                "BF16",
                "FP4_BLOCKSCALED_MM",
                "FP4_MX_MOE",
            ]
        ]
    ] = FieldInfo(alias="supportedPrecisions", default=None)

    supported_precisions_with_calibration: Optional[
        List[
            Literal[
                "PRECISION_UNSPECIFIED",
                "FP16",
                "FP8",
                "FP8_MM",
                "FP8_AR",
                "FP8_MM_KV_ATTN",
                "FP8_KV",
                "FP8_MM_V2",
                "FP8_V2",
                "FP8_MM_KV_ATTN_V2",
                "NF4",
                "FP4",
                "BF16",
                "FP4_BLOCKSCALED_MM",
                "FP4_MX_MOE",
            ]
        ]
    ] = FieldInfo(alias="supportedPrecisionsWithCalibration", default=None)

    supports_image_input: Optional[bool] = FieldInfo(alias="supportsImageInput", default=None)
    """If set, images can be provided as input to the model."""

    supports_lora: Optional[bool] = FieldInfo(alias="supportsLora", default=None)
    """Whether this model supports LoRA."""

    supports_serverless: Optional[bool] = FieldInfo(alias="supportsServerless", default=None)
    """If true, the model has a serverless deployment."""

    supports_tools: Optional[bool] = FieldInfo(alias="supportsTools", default=None)
    """If set, tools (i.e.

    functions) can be provided as input to the model, and the model may respond with
    one or more tool calls.
    """

    teft_details: Optional[object] = FieldInfo(alias="teftDetails", default=None)
    """TEFT addon details. Required if kind is HF_TEFT_ADDON.

    Must not be set otherwise.
    """

    training_context_length: Optional[int] = FieldInfo(alias="trainingContextLength", default=None)
    """The maximum context length supported by the model."""

    tunable: Optional[bool] = None
    """If true, the model can be fine-tuned.

    The value will be true if the tunable field is true, and the model is validated
    against the model_type field.
    """

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the model."""

    use_hf_apply_chat_template: Optional[bool] = FieldInfo(alias="useHfApplyChatTemplate", default=None)
    """
    If true, the model will use the Hugging Face apply_chat_template API to apply
    the chat template.
    """
